# Google::Search
# $Id: google.rb,v 1.5 2002/04/11 21:33:22 ianmacd Exp $
# 
# Version : 0.1.0
# Author  : Ian Macdonald <ian@caliban.org>
#
# Copyright (c) 2002 Ian Macdonald
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2, or (at your option)
#   any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software Foundation,
#   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

=begin

= NAME
  Google::Search
= SYNOPSIS
  require 'google'

  KEY = 'shajhdkhfjdfhjdfdjfdjfhdsgdhsgdg'

  query = ARGV.shift || 'ruby programming language'
  google = Google::Search.new(KEY)

  i = 0
  q = google.search(query)
  q.response['results'].each do |result|
    printf "\nResult # %d\n\n", i += 1
    result.each do |key|
      printf("%s = %s\n", key, result.attribute[key])
    end
  end

  puts '---------------------------------'
  printf "Estimated number of results is %d.\n",
          q.response['estimatedTotalResultsCount']
  printf "Your query took %6f seconds.\n", q.response['searchTime']

= DESCRIPTION
Google::Search allows you to programmatically query the Google search-engine
((<at www.google.com|URL:http://www.google.com>)). It is currently in the
alpha stage and the interface is liable to change at any time.
= CLASS METHODS
--- Search.new(key)
    This constructs a new Google search object. The key parameter is the
    key that Google assigned to you when you registered for the Web API
    download. If you don't yet have a key, go to
    ((<Google|URL:https://www.google.com/accounts/NewAccount?continue=http://api.google.com/createkey&followup=http://api.google.com/createkey>))
    and obtain one.
= INSTANCE METHODS
--- Search#search(query, start, max, filter, restrict, safe, lr, ie, oe)
    This performs a standard Google query search. Only the ((|query|))
    parameter is mandatory.

    The meaning of the other parameters can be obtained from section 2.1 of
    the APIs_Reference.html file that came with your Google API archive.

    This method returns a Hash, the keys of which are described in section
    3.1 of APIs_Reference.html.

    The ((|resultElements|)) element is, itself, a Hash, the elements of
    which are described in section 3.2 of APIs_Reference.html.
--- Search#spell(phrase)
    This attempts to perform a Google spell-check on ((|phrase|)). If Google
    has no spelling suggestions to make, ((*nil*)) is returned. Otherwise,
    a String is returned.
--- Search#cache(url)
    This attempts to retrieve the cached Google copy of the page corresponding
    to ((|url|)). If Google has not cached the URL in question, a page
    containing a message to this effect will be returned by the search-engine.

    This method always returns a String.
= AUTHOR
  Written by Ian Macdonald <ian@caliban.org>.
= COPYRIGHT
  Copyright (c) 2002 Ian Macdonald
  This is free software; see the source for copying conditions.
  There is NO warranty; not even for MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.
= SEE ALSO
  http://www.caliban.org/ruby
  http://www.google.com/
= BUGS
  Send all bug reports, enhancement requests and patches to the
  author.
= HISTORY
  $Id: google.rb,v 1.5 2002/04/11 21:33:22 ianmacd Exp $
=end

require 'soap/driver'

module Google

VERSION  = '0.1.0'
ENDPOINT = 'http://api.google.com/search/beta2'
NS = 'urn:GoogleSearch'

class Search
  attr_reader :response
  @@soap = SOAP::Driver.new(nil, nil, NS, ENDPOINT)

  def initialize(k)
    @key = k
    @response = Hash.new
  end

  def search(query, start=0, max=10, filter=false, restrict=nil, safe=false,
	     lr=nil, ie='latin1', oe='latin1')
    @@soap.addMethodWithSOAPAction(
      'doGoogleSearch', NS, 'key', 'q', 'start', 'maxResults', 'filter',
      'restrict', 'safeSearch', 'lr', 'ie', 'oe')
    r = @@soap.doGoogleSearch(
      @key, query, start, max, filter, restrict, safe, lr, ie, oe)
    @response['directoryCategories'] = r.directoryCategories
    @response['documentFiltering'] = r.documentFiltering
    @response['endIndex'] = r.endIndex
    @response['estimatedTotalResultsCount'] = r.estimatedTotalResultsCount
    @response['estimateIsExact'] = r.estimateIsExact
    @response['searchComments'] = r.searchComments
    @response['searchQuery'] = r.searchQuery
    @response['searchTime'] = r.searchTime
    @response['searchTips'] = r.searchTips
    @response['startIndex'] = r.startIndex
    @response['resultElements'] = Array.new
    r.resultElements.each do |e|
      @response['resultElements'].push(Google::ResultElement.new(e))
    end
    self
  end

  def spell(phrase)
    @@soap.addMethodWithSOAPAction('doSpellingSuggestion', NS, 'key',
				   'phrase')
    r = @@soap.doSpellingSuggestion(@key, phrase)
    r if r.type == String
  end

  def cache(url)
    @@soap.addMethodWithSOAPAction('doGetCachedPage', NS, 'key', 'phrase')
    # decode Base64-encoded response
    @@soap.doGetCachedPage(@key, url).unpack("m")
  end

end

class ResultElement
  attr_reader :attribute

  def initialize(element)
    @attribute = Hash.new
    @attribute['url'] = element.URL
    @attribute['snippet'] = element.snippet
    @attribute['title'] = element.title
    @attribute['cachedSize'] = element.cachedSize
    @attribute['relatedInformationPresent'] = element.relatedInformationPresent

    if element.directoryTitle.type == String
      @attribute['directoryTitle'] = element.directoryTitle
    end
    if element.summary.type == String
      @attribute['summary'] = element.summary
    end
    if element.hostName.type == String
      @attribute['hostName'] = element.hostName
    end
    if element.directoryCategory.fullViewableName.type == String
      @attribute['directoryName'] = element.directoryCategory.fullViewableName
    end
    if element.directoryCategory.specialEncoding == String
      @attribute['directoryEncoding'] = element.directoryCategory.specialEncoding
    end
  end
  private :initialize

  def each
    @attribute.each_key { |key| yield(key) }
  end
end

end
